/*******************************************************************************

  Dual Termometer 

  C/ PIC 16F676


  Autor : Cludio Lrios

  Data:   07/03/2017

  Aplicao: Termometro com visualizao de 2 temperaturas (2 sensores LM35).
  Uso em refrigeradores com freeser (temperatura do freeser) e do compartimento
  da geladeira (temperatura da geladeira).
  Poder tambm ser usado para indicar a temperatura interna e externa de uma
  residncia.
  
  Faixa de operao: de aproximadamente -50 a +150 C

  Uso didtico

  Este arquivo  parte integrante do blog Lrios.Tecnologia.WS


*******************************************************************************/
#include <16F676.h>
#device adc=10//adc com 10 bits
#use delay(clock=4000000)//frequencia interna
#fuses NOWDT,INTRC_IO, NOPROTECT, BROWNOUT, NOMCLR, NOCPD, PUT//configuraes

//byte
#byte  porta  = 0x05
#byte  portc  = 0x07
#byte  trisa  = 0x85
#byte  trisc  = 0x87

//pinos usados para comunicar com o LCD
#bit   en     = 0x07.4 //rc4  lcd 'enable' pino 6
#bit   rs     = 0x07.5 //rc5  lcd 'rs' - comando/dado pino 5
#bit   db4    = 0x07.0 //rc0  db4 pino 10
#bit   db5    = 0x07.1 //rc1  db5 pino 9
#bit   db6    = 0x07.2 //rc2  db6 pino 8
#bit   db7    = 0x07.3 //rc3  db7 pino 7

//pinos usados para comunicar com o LCD (tris_state)
#bit   tris_en     = 0x87.4 //rc4  lcd 'enable' pino 6
#bit   tris_rs     = 0x87.5 //rb3  lcd 'rs' - comando/dado pino 5
#bit   tris_db4    = 0x87.0 //rb4  dir db4 pino 10
#bit   tris_db5    = 0x87.1 //rb5  dir db5 pino 9
#bit   tris_db6    = 0x87.2 //rb6  dir db6 pino 8
#bit   tris_db7    = 0x87.3 //rb7  dir db7 pino 7

//defines
#define FATOR_CONV 19550 //para vref = 2,00 v -->20.000.000/1023=19550
#define VALOR_FILTRO 10  //filtro

//variaveis globais
unsigned int1 pos0,pos1,f_z;//flags de sinal e zero a esquerda
unsigned int8 a,ct0=0,ct1=0;//registradores para o filtro
unsigned int16 res_0,res_1,res_2,val_0,val_1;//resultado da converso
unsigned int16 val_old_0=0,val_old_1=0;//resultados anteriores
//==============================================================================
//                   ROTINAS PARA LCD
//==============================================================================
//clock para lcd
void lcd_clk(void){
 en=1;
 delay_us(10);//
 en=0;

 }

//envia nible para lcd
 void env_nib(int8 b){

 if(b&0x10)db4=1; else db4=0;
 if(b&0x20)db5=1; else db5=0;
 if(b&0x40)db6=1; else db6=0;
 if(b&0x80)db7=1; else db7=0;
 lcd_clk();
 }

 //envia comando para lcd
 void env_lcd_c(int8 val){
 rs=0;
 env_nib(val);
 env_nib(swap(val));
 delay_ms(10);
}

//envia dados para lcd
 void env_lcd_d(int8 val){
 rs=1;
 env_nib(val);
 env_nib(swap(val));
 }

//inicializao do lcd
void lcd_init(void){
 int8 a;

  rs=0;
  tris_en=0;//direo dos pinos como saida
  tris_rs=0;
  tris_db4=0;
  tris_db5=0;
  tris_db6=0;
  tris_db7=0;
  db4=0;//situao inicial resetados
  db5=0;
  db6=0;
  db7=0;
  en=0;
  delay_ms(50);//delay inicial
  db4=1; db5=1;// setados
  for(a=0;a<3;a++) {lcd_clk(); delay_ms(5);}// envia 3,3,3
  db4=0;//envia 2
  lcd_clk();//pulso em 'en'
  delay_ms(5);
  env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
  env_lcd_c(0x06);// no shift
  env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  env_lcd_c(0x01);//limpa tela
  delay_ms(10);

}


/*******************************************************************************
       Rotina de apresentao dos resultados no LCD
*******************************************************************************/
void mostra(unsigned int16 dado,unsigned int1 sign){

  unsigned int32 val_x;//registrador para a matemtica
  unsigned int8  res[]={' ',0,0,0,',',0};//resultados
  unsigned int8 a;
 
  val_x=(int32)dado;
  val_x=val_x * FATOR_CONV; //converte valor lido para valor da temperatura
  f_z=0;//reseta indicador de zero a esquerda
  while(val_x>=10000000) {val_x=val_x-10000000;res[1]++;}//centena
  res[1]|='0';//normaliza ASCII
  if(res[1]=='0'){f_z=1; res[1]=' ';} //elimina zeros a esquerda
  while(val_x>=1000000) {val_x=val_x-1000000;res[2]++;}//dezena
  res[2]|='0';//normaliza ASCII
  if((res[2]=='0')&&(f_z))res[2]=' ';//elimina zeros a esquerda
  while(val_x>=100000) {val_x=val_x-100000;res[3]++;}//unidade
  res[3]|='0';//normaliza ASCII
  while(val_x>=10000) {val_x=val_x-10000;res[5]++;}//casa ps virgula
  res[5]|='0';//normaliza ASCII
  if(!sign)res[1]='-';//mostra sinal '-' se for abaixo de zero graus
  

  for(a=0;a<6;a++){ env_lcd_d(res[a]); }//envia para o lcd o resultado
  env_lcd_d(0xdf);//simbolo de graus
  env_lcd_d('C');//letra 'C'

}

//-------------------------FIM DAS ROTINAS DE LCD--------------------------------


/****************************************************************************
                         Rotina Main
****************************************************************************/
void main() {

   setup_adc_ports(sAn0|sAn3|sAn2|VSS_VREF);//define entradas do conversor AD
   setup_adc(ADC_CLOCK_INTERNAL);//frequncia da converso
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   trisc=0;//tudo sada
   portc=0;//desliga todas as sadas
   trisa=0xff; //tudo entrada
   Lcd_Init(); //Inicializa o display
   printf(env_lcd_d,"Dual Termometer ");// nome do aparelho
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d,"Larios.Tecnol.Ws");//blog
   delay_ms(2000);//tempo de apresentao
   env_lcd_c(1);//limpa tela


   /****************************************************************************
                         Lao Principal
   ****************************************************************************/
   for(;;){

   set_adc_channel( 0 );//ajusta para canal 0
   delay_us(60);//tempo para estabilizar entrada
   res_0=0;
   for(a=0;a<32;a++){ res_0 += read_adc();}
   res_0/=32;


   set_adc_channel( 3 );//ajusta para canal 3
   delay_us(60);//tempo para estabilizar entrada
   res_1=0;
   for(a=0;a<32;a++){ res_1 += read_adc();}
   res_1/=32;

   set_adc_channel( 2 );//ajusta para canal 2
   delay_us(60);//tempo para estabilizar entrada
   res_2=0;
   for(a=0;a<32;a++){ res_2 += read_adc();}
   res_2/=32;


   /****************************************************************************
        Subtrai da tenso medida nos LM35 o valor medido nos diodos
   ****************************************************************************/

   if(res_0>=res_2){//canal 0
   val_0=(res_0 - res_2);
   pos0=1;
   }
   else{//caso temperatura seja 'negativa'
   val_0=(res_2 - res_0);
   pos0=0;
   }

   if(res_1>=res_2){//canal 3
   val_1=(res_1 - res_2);
   pos1=1;//positiva
   }
   else{//caso temperatura seja 'negativa'
   val_1=(res_2 - res_1);
   pos1=0;//negativa
   }
/*******************************************************************************
          Filtra oscilaes indesejveis
*******************************************************************************/
   if(val_0!=val_old_0){if(++ct0>VALOR_FILTRO){ct0=0; val_old_0=val_0;}}//filtra oscilaes canal 0
   else{ct0=0;}

   if(val_old_1!=val_1){if(++ct1>VALOR_FILTRO){ct1=0; val_old_1=val_1;}}//filtra oscilaes canal 3
   else{ct1=0;}
/*******************************************************************************
          Mostra resultado no LCD
*******************************************************************************/

   env_lcd_c(0x80);//vai para comeo da linha 1
   printf(env_lcd_d,"Temp_A =");// mostra 'TempA ="
   mostra(val_old_0,pos0);//mostra resultado do LM35 ligado ao canal 0
   env_lcd_c(0xc0);//vai para o comeo da linha 2
   printf(env_lcd_d,"Temp_B =");// mostra 'TempB ="
   mostra(val_old_1,pos1);//mostra resultado do LM35 ligado ao canal 3
   delay_ms(30);//delay

   }//for(;;)

}//main()
